<?php

include_once 'ubrr.core/_autoload.php';

class WordpressPaymentMailNotification extends UBRIR\MailNotifications
{
    private function getAndSaveWordpressOption($view_data_key, $wp_option_name)
    {
        if (!is_string($view_data_key) || !is_string($wp_option_name)) {
            return false;
        }

        if (!isset($this->viewData[$view_data_key])) {
            $this->viewData[$view_data_key] = boolval(get_option($wp_option_name));
        }

        return $this->viewData[$view_data_key];
    }

    public function GetDefaultReceiversEmail()
    {
        $adminEmail = get_option('admin_email');

        return $adminEmail;
    }

    public function customMail($to, $subject, $message, $headers)
    {
        return wp_mail($to, $subject, $message, $headers);
    }

    public function ConnectResultViewer(\UBRIR\PaymentResultViewer $resultViewer)
    {
        $this->resultViewer = $resultViewer;
    }

    public function isNotificationsEnabled()
    {
        return $this->getAndSaveWordpressOption('is_notifications_enabled', 'ubrr_notifications_y');
    }

    //region Проверка необходимости добавлять блок
    public function isNeedItSiteURL()
    {
        return $this->getAndSaveWordpressOption('is_need_site_url', 'ubrr_shop_site_y');
    }

    public function isNeedItShopEmail()
    {
        return $this->getAndSaveWordpressOption('is_need_shop_email', 'ubrr_shop_email');
    }

    public function isNeedItOrderAmount()
    {
        return $this->getAndSaveWordpressOption('is_need_order_amount', 'ubrr_amount_y');
    }

    public function isNeedItOrderDate()
    {
        return $this->getAndSaveWordpressOption('is_need_order_date', 'ubrr_date_y');
    }

    public function isNeedItOrderNumber()
    {
        return $this->getAndSaveWordpressOption('is_need_order_number', 'ubrr_ordernum_y');
    }

    public function isNeedItCardNumber()
    {
        return $this->getAndSaveWordpressOption('is_need_card_number', 'ubrr_cardnum_y');
    }

    public function isNeedItCardHolderName()
    {
        return $this->getAndSaveWordpressOption('is_need_card_holder_name', 'ubrr_cardholder_y');
    }

    public function isNeedItOperationType()
    {
        return $this->getAndSaveWordpressOption('is_need_operation_type', 'ubrr_opertype_y');
    }
    //endregion

    //region Получение значений
    public function GetSiteURL()
    {
        if (!isset($this->viewData['site_url'])) {
            $this->viewData['site_url_href'] = get_site_url();
            $this->viewData['site_url_title'] = get_bloginfo('name');

            $this->viewData['site_url'] = '<a href="'.$this->viewData['site_url_href'].'">'.$this->viewData['site_url_title'].'</a>';
        }

        return $this->viewData['site_url'];
    }

    public function GetLogoURL()
    {
        if (!isset($this->viewData['logo_url'])) {
            return $this->viewData['logo_url'] = get_site_url(
                ).'/wp-content/plugins/ubrr/ubrr.core/views/ubrr_mail_logo.png';
        }
    }

    public function GetShopEmail()
    {
        if (!isset($this->viewData['v_shop_email'])) {
            $this->viewData['v_shop_email'] = get_option('admin_email');
        }

        return $this->viewData['v_shop_email'];
    }

    public function GetOrderAmount()
    {
        if ($this->resultViewer !== null) {
            return $this->resultViewer->GetPaymentAmount();
        }

        return 0;
    }

    public function GetOrderDateTime()
    {
        if ($this->resultViewer !== null) {
            return $this->resultViewer->GetPaymentTime();
        }

        return '';
    }

    public function GetOrderNumber()
    {
        if ($this->resultViewer !== null) {
            return $this->resultViewer->GetShopOrderId();
        }

        return 0;
    }

    public function GetCardHolderName()
    {
        if ($this->resultViewer !== null) {
            return $this->resultViewer->GetCardOwner();
        }

        return '';
    }

    public function GetCardNumber()
    {
        if ($this->resultViewer !== null) {
            return $this->resultViewer->GetCardPAN();
        }

        return '';
    }
    //endregion
}