<?php

include_once 'ubrr.core/_autoload.php';

use \UBRIR\InterfaceConnectionConfig,
    \UBRIR\WorkMode;

include_once 'WordpressPaymentMailNotification.php';

class WordpressUBRiRConnector implements InterfaceConnectionConfig
{
    public function UpdateCertificateCreateDate($creationDate = false)
    {
        update_option('ubrr_time_cert_created', date('d.m.Y', time()));
    }

    public function SetPrivateCertificatePassword($newPassword)
    {

    }

    public function SetNewPrivateCertificatePassword($newPassword)
    {
        update_option('new_certificate_password', $newPassword);
    }

    public function GetCertificateCreateDate()
    {
        $date = get_option('ubrr_time_cert_created');

        return $date;
    }

    public function GetNewPrivateCertificatePassword()
    {
        return get_option('new_certificate_password');
    }

    public function GetBankReceiversEmailList()
    {
        $emails = get_option('bank_email');
        $arRecievers = explode(',', $emails);

        return $arRecievers;
    }

    public function SetShopId($shopId)
    {
        update_option('shop_id', $shopId);
    }

    public function GetWorkMode()
    {
        $shopId = self::GetShopId();

        $workMode = new WorkMode(WorkMode::Test);

        if ($shopId !== null) {
            $test_mode = get_option('test_mode');

            if ($test_mode != 1) {
                $workMode = new WorkMode(WorkMode::Normal);
            }
        }

        return $workMode;
    }

    public function GetShopId()
    {
        $shopId = get_option('shop_id');

        return $shopId;
    }

    public function GetPrivateCertificatePassword()
    {
        $shopId = self::GetShopId();

        if ($shopId !== null) {
            $privatePassword = get_option('certificate_password');

            return $privatePassword;
        }

        return false;
    }

    public function GetOrderDataByShopOrderID($OrderId)
    {
        global $wpdb;
        $table_name = $wpdb->prefix."ubrr_twpg_orders";
        $sql = 'SELECT * FROM `'.$table_name.'` WHERE `wp_id`='.$OrderId;
        $rows = $wpdb->get_results($sql, ARRAY_A);

        if (count($rows) >= 1) {
            $row = $rows[0];

            $orderID = $row['wp_id'];
            $TWPG_OrderId = $row['twpg_id'];
            $SessionId = $row['session_id'];

            return array(
                'order_id' => $orderID,
                'twpg_id' => $TWPG_OrderId,
                'session_id' => $SessionId,
            );
        }

        return false;
    }

    public function GetOrderDataByUBRIROrderID($TWPG_OrderId)
    {
        global $wpdb;
        $table_name = $wpdb->prefix."ubrr_twpg_orders";
        $sql = 'SELECT * FROM `'.$table_name.'` WHERE `twpg_id`='.$TWPG_OrderId;
        $rows = $wpdb->get_results($sql, ARRAY_A);

        if (count($rows) >= 1) {
            $row = $rows[0];

            $orderID = $row['wp_id'];
            $TWPG_OrderId = $row['twpg_id'];
            $SessionId = $row['session_id'];

            return array(
                'order_id' => $orderID,
                'twpg_id' => $TWPG_OrderId,
                'session_id' => $SessionId,
            );
        }

        return false;
    }

    //TODO: Убрать $ecomSessionID из этого вызова, если он прям нужен возможно даже лучше другую таблицу завести
    public function SaveTWPGOrder($orderId, $TWPG_OrderId, $SessionId, $ecomSessionID = '')
    {
        global $wpdb;
        $table_name = $wpdb->prefix."ubrr_twpg_orders";
        $sql = "INSERT INTO `".$table_name."` (`wp_id`,`twpg_id`,`session_id`,`ecom_session_id`)
                VALUES ('".$orderId."','".$TWPG_OrderId."','".$SessionId."','".$ecomSessionID."')
                ON DUPLICATE KEY UPDATE 
                `wp_id`='".$orderId."',
                 `twpg_id`='".$TWPG_OrderId."',
                  `session_id`='".$SessionId."',
                  `ecom_session_id`='".$ecomSessionID."'";

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');
        $result = dbDelta($sql);

        return $result;
    }

    public function GetOrdersList($pageNum = 1, $perPageCount = 0)
    {
        $perPageCount = intval($perPageCount);

        global $wpdb;
        $table_name = $wpdb->prefix."ubrr_twpg_orders";
        $sql = 'SELECT * FROM `'.$table_name.'` ORDER BY wp_id DESC';

        //TODO: добавить выбор страницы
        if ($perPageCount > 0) {
            $sql .= ' LIMIT '.$perPageCount;
        }

        $ordersList = Array();
        $dbOrdersList = $wpdb->get_results($sql, ARRAY_A);

        foreach ($dbOrdersList as $row) {
            $ordersList[] = array(
                'order_id' => $row['wp_id'],
                'twpg_id' => $row['twpg_id'],
            );
        }

        return $ordersList;
    }

    public function GetDefaultContinueLink()
    {
        return get_site_url();
    }

    public function GetMailNotifier()
    {
        $mailNotification = new WordpressPaymentMailNotification();

        return $mailNotification;
    }


}