<?php

namespace UBRIR;

/**
 * Магические обертки вокруг GetOrders()
 * @method ShowOrders($pageNum, $perPage) Вывод списка заказов в html виде
 * @method ReturnHTMLOrders($pageNum, $perPage) Возвращение html шаблона списка заказов
 * @method JsonOrders($pageNum, $perPage) Списка заказов в json формате
 * @method HtmlInJsonOrders($pageNum, $perPage) Списков заказов в json в том числе шаблон внутри JSON
 *
 * Магические обертки вокруг GetOrderStatus()
 * @method ShowOrderStatus($orderId) Вывод статуса заказа
 * @method ReturnHTMLOrderStatus($orderId) Возвращение html шаблона статуса заказа
 * @method JsonOrderStatus($orderId) Статус заказа в json формате
 * @method HtmlInJsonOrderStatus($orderId) Статус заказа в json вместе с шаблоном
 *
 *
 * Магические обертки вокруг GetOrderInfo()
 * @method ShowOrderInfo($orderId) Вывод информации о заказе
 * @method ReturnHTMLOrderInfo($orderId) Возвращение html шаблона информации о заказе
 * @method JsonOrderInfo($orderId) Информация о заказе в json формате
 * @method HtmlInJsonOrderInfo($orderId) Информация о заказе в json вместе с шаблоном
 *
 * Магические обертки вокруг GetReconcile()
 * @method ShowReconcile
 * @method HtmlInJsonReconcile
 *
 * @method ShowJournal
 *
 * Магические обертки вокруг GetLog()
 * @method ShowLog()
 * @methid ReturnHTMLLog()
 * @JsonLog()
 * @HtmlInJsonLog()
 *
 */
abstract class GatewayForAdmin extends BitwiseFlag
{
    protected $FLAG_HTML;
    protected $FLAG_JSON_RESULT;
    protected $FLAG_HTML_PRINT;
    protected $FLAG_JSON_WITH_HTML;
    protected $FLAG_HTML_RETURN;

    const DEFAULT_PREFIX = 'Get';
    const JSON_RESULT_PREFIX = 'Json';
    const HTML_RESULT_PREFIX = 'Show';
    const HTML_RETURN_PREFIX = 'ReturnHTML';
    const JSON_WITH_HTML_RESULT_PREFIX = 'HtmlInJson';

    static $__callCache = null;
    private $_gatewayPanel;
    private $_functionBody = '';
    private $_connector = null;

    function __construct(InterfaceConnectionConfig $connector = null)
    {
        $this->FLAG_HTML = 1;
        $this->FLAG_JSON_RESULT = 2;
        $this->FLAG_HTML_PRINT = 4 | $this->FLAG_HTML;
        $this->FLAG_JSON_WITH_HTML = $this->FLAG_HTML | $this->FLAG_JSON_RESULT;
        $this->FLAG_HTML_RETURN = 8 | $this->FLAG_HTML;

        $this->_connector = $connector;
    }

    public function GetGatewayPanel($useLogs = true)
    {
        if ($this->_gatewayPanel === null) {
            $this->_gatewayPanel = new GatewayInteractionPanel($this->_connector, $useLogs);
        }

        return $this->_gatewayPanel;
    }

    /**
     * Создание виртуальных оберток вокруг методов Get* которые в зависимости от приставки Show*\GetJSON_* возвращают результат в определенном формате
     *
     * @param $functionName
     * @param $arguments
     * @return mixed|null
     */
    public function __call($functionName, $arguments)
    {
        $var_0 = null;
        $var_1 = null;
        $var_2 = null;

        if (count($arguments) == 1 && is_array($arguments[0])) {
            //Разворачивание аргументов если мы передавали их по цепочке
            $arguments = $arguments[0];

            //PHP >5.6 можно разворачивать аргументы через ...$arguments при передаче дальше по цепочке
            extract($arguments, EXTR_PREFIX_ALL, 'var');
        } else {
            extract($arguments, EXTR_PREFIX_ALL, 'var');
        }

        if (static::$__callCache == null) {
            $this->_fillCallCache();
        }

        if (isset(static::$__callCache[$functionName])) {
            $getFunctionName = static::$__callCache[$functionName];

            if (strpos($functionName, self::JSON_RESULT_PREFIX) === 0) {
                $this->setFlag($this->FLAG_JSON_RESULT);
            } elseif (strpos($functionName, self::JSON_WITH_HTML_RESULT_PREFIX) === 0) {
                $this->setFlag($this->FLAG_JSON_WITH_HTML);
            } elseif (strpos($functionName, self::HTML_RESULT_PREFIX) === 0) {
                $this->setFlag($this->FLAG_HTML_PRINT);
            } elseif (strpos($functionName, self::HTML_RETURN_PREFIX) === 0) {
                $this->setFlag($this->FLAG_HTML_RETURN);
            } else {
                return null;
            }

            //Получаем имя реализованной функции для вызова
            $this->_functionBody = substr($getFunctionName, strlen(self::DEFAULT_PREFIX));

            $resultDataContainer = $this->$getFunctionName(${'var_0'}, ${'var_1'}, ${'var_2'});

            return $this->PrepareCallResult($resultDataContainer);
        }

        return null;
    }

    private function PrepareCallResult(DataContainer $resultDataContainer)
    {
        $viewDir = $this->GetGatewayPanel()->GetViewsDir();

        //Обработка HTML шаблона
        if ($this->isFlagSet($this->FLAG_HTML)) {
            global $view_data;
            $view_data = $resultDataContainer->GetResultArray();

            $templateFile = $viewDir.'ubrr.view.'.strtolower($this->_functionBody).'.php';

            if ($resultDataContainer->isSuccess() && file_exists($templateFile)) {

                //Буферизация вывода шаблона
                ob_start();
                include_once($templateFile);
                $html_output = ob_get_clean(); //ob_get_contents + ob_end_clean

                $resultDataContainer->SetResultRecord('html', $html_output);
            } else {
                $view_data['ERRORS'] = $resultDataContainer->GetErrors();

                $templateFile = $viewDir.'ubrr.view.error.php';

                ob_start();
                include_once($templateFile);
                $html_output = ob_get_clean(); //ob_get_contents + ob_end_clean

                $resultDataContainer->SetResultRecord('html', $html_output);
            }
        }

        //Вывод шаблона в консоль
        if ($this->isFlagSet($this->FLAG_HTML_PRINT)) {
            $arResult = $resultDataContainer->GetResultArray();
            echo $arResult['html'];
        }
        if ($this->isFlagSet($this->FLAG_HTML_RETURN)) {
            $arResult = $resultDataContainer->GetResultArray();

            return $arResult['html'];
        }

        if ($this->isFlagSet($this->FLAG_JSON_RESULT)) {
            header('Content-Type: application/json');
            echo json_encode($resultDataContainer->GetResultArray());
        }

        return $resultDataContainer;
    }

    protected function _fillCallCache()
    {
        static::$__callCache = Array();

        $methodsList = get_class_methods(__CLASS__);
        foreach ($methodsList as $methodName) {
            if (substr($methodName, 0, 3) == 'Get') {
                $methodNameBody = substr($methodName, 3);

                //static::$__callCache[self::DATA_CONTAINER_RESULT_PREFIX . $methodNameBody] = $methodName;
                static::$__callCache[self::HTML_RESULT_PREFIX.$methodNameBody] = $methodName;
                static::$__callCache[self::JSON_WITH_HTML_RESULT_PREFIX.$methodNameBody] = $methodName;
                static::$__callCache[self::JSON_RESULT_PREFIX.$methodNameBody] = $methodName;
                static::$__callCache[self::HTML_RETURN_PREFIX.$methodNameBody] = $methodName;
            }
        }
    }

    //region Методы с ВОЛШЕБНЫМИ собратьями :)
    public function GetOrders($pageNum = 1, $perPageCount = 0)
    {
        $arResult = $this->GetGatewayPanel()->GetOrdersList($pageNum, $perPageCount);
        $resultDataContainer = new DataContainer();
        $resultDataContainer->SetResultRecord('order_list', $arResult);
        $resultDataContainer->setSuccess(); //Пустой результирующий массив это не ошибка!

        return $resultDataContainer;
    }

    /**
     * @param int $TWPG_OrderId
     * @return null|DataContainer
     */
    public function GetOrderStatus($TWPG_OrderId = -1)
    {
        if ($TWPG_OrderId > 0) {
            $orderRow = $this->GetGatewayPanel()->GetOrderDataByUBRIROrderID($TWPG_OrderId);
            $TWPG_SessionId = $orderRow['session_id'];

            $resultDataContainer = $this->GetGatewayPanel()->SetupAndCheckOrderStatus($TWPG_OrderId, $TWPG_SessionId);
            $orderStatusCode = $resultDataContainer->getFieldValue('OrderStatus');
            $orderStatus = TypeEnums::GetLabel_OrderStatus($orderStatusCode);

            $resultDataContainer->SetResultRecord('order_status', Array('code' => $orderStatusCode))
                ->SetResultRecord('label', $orderStatus);

            if (!$resultDataContainer->isSuccess()) {
                $shopId = $this->GetGatewayPanel()->getShopId();
                $operation = $resultDataContainer->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $resultDataContainer->AddError('Заказ был оформлен с другим номером ИМ');
                    $resultDataContainer->SetResultRecord('order_status', Array('code' => $orderStatusCode))
                        ->SetResultRecord('label', 'Заказ был оформлен с другим номером ИМ');

                }
            }
        } else {
            $resultDataContainer = new DataContainer();
            $resultDataContainer->AddError('Недостаточно параметров для выполнения запроса');
        }

        return $resultDataContainer;
    }

    /**
     * @param int $TWPG_OrderId
     * @return null|DataContainer
     */
    public function GetOrderInfo($TWPG_OrderId = -1)
    {
        if ($TWPG_OrderId > 0) {
            $orderRow = $this->GetGatewayPanel()->GetOrderDataByUBRIROrderID($TWPG_OrderId);
            $TWPG_SessionId = $orderRow['session_id'];

            $responseData = $this->GetGatewayPanel()->SetupAndGetOrderInformation($TWPG_OrderId, $TWPG_SessionId);
            if (!$responseData->isSuccess()) {
                $shopId = $this->GetGatewayPanel()->getShopId();
                $operation = $responseData->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $responseData->AddError('Заказ был оформлен с другим номером ИМ');
                }
            } else {
                $parsedResponse = $responseData->GetParsedResponse();

                $responseData->SetResultRecord('order_info', $parsedResponse['orderinfo']);
            }
        } else {
            $responseData = new DataContainer();
            $responseData->AddError('Недостаточно параметров для выполнения запроса');
        }

        return $responseData;
    }

    public function GetReconcile()
    {
        try {
            $responseData = $this->GetGatewayPanel()->Reconcile();

            if (!$responseData->isSuccess()) {
                $shopId = $this->GetGatewayPanel()->getShopId();
                $operation = $responseData->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $responseData->AddError('Заказ был оформлен с другим номером ИМ');
                }
            }
        } catch (\Exception $e) {
            $responseData = new DataContainer();
            $responseData->AddError($e->getMessage());
        }

        return $responseData;
    }

    public function GetJournal()
    {
        try {
            $responseData = $this->GetGatewayPanel()->GetJournal();

            if (!$responseData->isSuccess()) {
                $shopId = $this->GetGatewayPanel()->getShopId();
                $operation = $responseData->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $responseData->AddError('Заказ был оформлен с другим номером ИМ');
                }
            } else {
                $responseData->CopyFieldsToResultArray();
            }
        } catch (\Exception $e) {
            $responseData = new DataContainer();
            $responseData->AddError($e->getMessage());
        }

        return $responseData;
    }

    public function GetLog()
    {
        $responseData = new DataContainer();
        $logRecords = $this->GetGatewayPanel(false)->GetDebugLogRecords();

        $logRecords = $this->FormatLogErrors($logRecords);
        $responseData->SetResultRecord('log_records', $logRecords);
        $responseData->setSuccess();

        return $responseData;
    }


    //endregion

    //region Проброс методов из GatewayInteractionPanel
    public function ClearLog()
    {
        return $this->GetGatewayPanel()->CleanLog();
    }

    protected function FormatLogErrors($logRecords)
    {
        foreach ($logRecords as $key => $record) {
            $timeLineEnd = strpos($record, "\r\n") + strlen("\r\n");
            $timeLine = substr($record, 0, $timeLineEnd);

            $errorsBlock = '';
            $firstErrorLineStart = strpos($record, "\tERROR");
            if ($firstErrorLineStart !== false) {
                $lastErrorLineStart = strrpos($record, "\tERROR");

                $lastErrorLineEnd = strpos($record, "\r\n", $lastErrorLineStart);

                $errorsBlock = substr(
                    $record,
                    $firstErrorLineStart,
                    ($lastErrorLineEnd - $firstErrorLineStart) + strlen("\r\n")
                );
                $errorsBlock = str_replace(
                    Array("\tERROR", "\r\n"),
                    Array('<span class="errorLine">ERROR', "</span>\r\n"),
                    $errorsBlock
                );

                $recordText = substr($record, $timeLineEnd, $firstErrorLineStart - $timeLineEnd).substr(
                        $record,
                        $lastErrorLineEnd
                    );
            } else {
                $recordText = substr($record, $timeLineEnd);
            }

            $haveEndTag = false;
            if (strrpos($recordText, '[END]') !== false) {
                $haveEndTag = true;
                $recordText = str_replace('[END]', '', $recordText);
            }

            $logRecords[$key] = array(
                'start_time' => $timeLine,
                'text' => $recordText,
                'errors' => $errorsBlock,
                'success_end' => $haveEndTag,
            );
        }

        return $logRecords;
    }

    //endregion

    public function ConnectToCMS(InterfaceConnectionConfig $connector)
    {
        $this->GetGatewayPanel()->ConnectToCMS($connector);

        return $this;
    }

    /**
     * @return DataContainer
     */
    public static function ReturnEmptyResult()
    {
        $emptyDataContainer = new DataContainer();

        return $emptyDataContainer;
    }
}