<?php

namespace UBRIR;

class MailNotifications
{
    var $fromName = 'Платежный шлюз УБРиР';

    protected $viewData;
    /** @var PaymentResultViewer */
    protected $resultViewer;

    public static function UTF8_Encode($CyrillicString)
    {
        return '=?utf-8?b?'.base64_encode($CyrillicString).'?=';
    }

    public function ConnectResultViewer(PaymentResultViewer $resultViewer)
    {
        $this->resultViewer = $resultViewer;
    }

    public function GetDefaultReceiversEmail()
    {
        return '';
    }

    public function SendPaymentNotification($receiverEmail = '', $receiverName = '')
    {
        //Потенциально несколько получаталей
        //$arTo = Array('Александр Лысенко' => 'ya.lysenkoalexander@yandex.ru');
        if (empty($receiverEmail)) {
            $receiverEmail = $this->GetDefaultReceiversEmail();
        }
        if (empty($receiverEmail)) {
            return false;
        }

        if (!is_array($receiverEmail)) {
            if (!is_array($receiverName)) {
                $arTo = Array($receiverEmail => $receiverName);
            }
        } else {
            $arTo = $receiverEmail;
        }
        //$to = implode(', ', array_values($arTo));
        $to = '';
        foreach ($arTo as $name => $email) {
            if (is_string($name) && strlen($name) > 0) {
                $to .= self::UTF8_Encode($name).' <'.$email.'>, ';
            } else {
                $to .= '<'.$email.'>, ';
            }
        }
        $to = substr($to, 0, strlen($to) - 2); //Удаляем последний [, ]

        // тема письма
        $subject = 'Уведомление о платеже';

        $curFileDir = dirname(realpath(__FILE__));
        // текст письма
        $mailTemplate = realpath($curFileDir.'/../views/ubrr.mail.adminnotification.php');

        if ($mailTemplate) {
            global $mailNotification;
            $mailNotification = $this;

            ob_start();
            include_once $mailTemplate;
            $message = ob_get_contents();
            ob_end_clean();

            // Для отправки HTML-письма должен быть установлен заголовок Content-type
            $headers = 'MIME-Version: 1.0'."\r\n";
            $headers .= 'Content-type: text/html; charset=UTF-8'."\r\n";
            //$headers .= 'From: '.self::UTF8_Encode($this->fromName).' <'.$this->fromEmail.'>'."\r\n";

            // Отправляем (сначала через штатный mail, потом через кастомный отправитель если штатный не работает)
            if (!mail($to, $subject, $message, $headers)) {
                return $this->customMail($receiverEmail, $subject, $message, $headers);
            } else {
                return true;
            }
        }

        return false;
    }

    public function customMail($to, $subject, $message, $headers)
    {
        return mail($to, $subject, $message, $headers);
    }

    public function isNotificationsEnabled()
    {
        return true;
    }

    //region Проверка необходимости добавлять блок
    public function isNeedItSiteURL()
    {
        return true;
    }

    public function isNeedItShopEmail()
    {
        return true;
    }

    public function isNeedItOrderAmount()
    {
        return true;
    }

    public function isNeedItOrderDate()
    {
        return true;
    }

    public function isNeedItOrderNumber()
    {
        return true;
    }

    public function isNeedItCardNumber()
    {
        return true;
    }

    public function isNeedItCardHolderName()
    {
        return true;
    }

    public function isNeedItOperationType()
    {
        return true;
    }
    //endregion

    //region Получение значений
    public function GetSiteURL()
    {
        if (isset($this->viewData['site_url'])) {
            return $this->viewData['site_url'];
        }

        return '<a href="">Ссылка на интернет-магазин</a>';
    }

    public function GetLogoURL()
    {
        if (isset($this->viewData['logo_url'])) {
            return $this->viewData['logo_url'];
        }

        return '';
    }

    public function GetShopEmail()
    {
        if (isset($this->viewData['v_shop_email'])) {
            return $this->viewData['v_shop_email'];
        }

        return '';
    }

    public function GetOrderAmount()
    {
        return 0;
    }

    public function GetOrderDateTime()
    {
        $currentDate = new \DateTime();

        return $currentDate->format('dd.mm.YY h:i:s');
    }

    public function GetOrderNumber()
    {
        return 0;
    }

    public function GetCardHolderName()
    {
        return '';
    }

    public function GetCardNumber()
    {
        return '';
    }
    //endregion
}