<?php

namespace UBRIR;

class NewCertificates
{
    private $gatewayPanel;

    public function __construct(GatewayInteractionPanel $panel)
    {
        $this->gatewayPanel = $panel;
    }

    /**
     * @param $merchantId
     * @param $managerEmail
     * @param $passPhrase
     * @param string $country
     * @param string $region
     * @param string $city
     * @param string $organization
     * @param string $division
     * @return DataContainer
     */
    function Generate(
        $merchantId,
        $managerEmail,
        $passPhrase = '',
        $country = 'RU',
        $region = 'SVO',
        $city = 'MSK',
        $organization = 'Some Organization',
        $division = 'Main'
    ) {
        $responseData = new DataContainer();
        $connector = $this->gatewayPanel->GetCMSConnector();

        $arReceivers = $connector->GetBankReceiversEmailList();

        if (!is_array($arReceivers)) {
            $responseData->AddError(
                'Неверный формат получателей почты. Можно использовать массив получателей или строку с разделитилем [,]'
            );

            return $responseData;
        }

        if (empty($merchantId)) {
            $responseData->AddError('Не указан идентификатор магазина (MerchantID)');

            return $responseData;
        }

        if (empty($managerEmail)) {
            $responseData->AddError('Неверный email ответственного лица');

            return $responseData;
        }

        if (empty($passPhrase)) {
            $responseData->AddError('Не указан пароль от сертификата. Укажите пароль');

            return $responseData;
        }


        //Запись $merchantId если текущий пустой
        $currentShopId = $connector->GetShopId();
        if (empty($currentShopId)) {
            $connector->SetShopId($merchantId);
        }

        //При генерации сохраняем новый пароль отдельного от работающего. После заливки .pem мы будем использовать его
        $connector->SetNewPrivateCertificatePassword($passPhrase);


        ob_start(); //Защищаем вывод от Warning'ов в процессе генерации
        //Генерируем приватный ключ
        $privateKeyPath = $this->getNewPrivateCertificateFilePath();
        $command = 'openssl genrsa -des3 -passout pass:'.escapeshellarg($passPhrase).' -out '.escapeshellarg(
                $privateKeyPath
            ).' 2048';
        $result = false;
        $output = '';
        exec($command, $output, $result);

        //$keys = openssl_pkey_new();

        //Проверка ключа
        if ($result) {
            $responseData->AddError('Возникла проблема с генерацией ключа');

            return $responseData;
        }

        //Читаем приватный ключ
        $privateKeyText = openssl_pkey_get_private('file://'.$privateKeyPath, $passPhrase);
        if ($privateKeyText === false || filesize($privateKeyPath) == 0) {
            $responseData->AddError('Не удалось сгенерировать новый приватный ключ');

            return $responseData;
        }

        //Вносим переменные в массив для генерации
        $requestData = array(
            "countryName" => $country,
            "stateOrProvinceName" => $region,
            "localityName" => $city,
            "organizationName" => $organization,
            "organizationalUnitName" => $division,
            "commonName" => $merchantId,
            "emailAddress" => $managerEmail,
        );

        //Генерируем запрос
        $request = openssl_csr_new($requestData, $privateKeyText, Array("digest_alg" => "sha256"));

        //Проверка запроса
        if (!$request) {
            $responseData->AddError('Возникла проблема с генерацией запроса');
        }

        //Помещаем содержимое запроса в файл
        $requestPath = $this->getRequestFilePath();
        openssl_csr_export($request, $requestText);
        $requestFile = fopen($requestPath, 'w');
        fwrite($requestFile, $requestText);
        fclose($requestFile);

        //Данные для отправки почты
        $from = $managerEmail;
        $subject = 'Запрос на создание сертификата';
        $subject = MailNotifications::UTF8_Encode($subject);
        $message = 'Файл запроса во вложении';

        //Имя вложенного файла
        $filename = 'request.csr';

        //Разделитель
        $boundary = "---";

        //Заголовки
        $headers = "From: $from\nReply-To: $from\n";
        $headers .= "Content-Type: multipart/mixed; boundary=\"$boundary\"";
        $body = "--$boundary\n";

        /* Присоединяем текстовое сообщение */
        $body .= "Content-type: text/html; charset=\"utf-8\"\n";
        $body .= "Content-Transfer-Encoding: quoted-printablenn";
        $body .= "Content-Disposition: attachment; filename==?utf-8?B?".base64_encode($filename)."?=\n\n";
        $body .= $message."\n";
        $body .= "--$boundary\n";

        //Считываем файл csr запроса
        $file = fopen($requestPath, "r");
        $text = fread($file, filesize($requestPath));
        fclose($file);

        // Добавляем тип содержимого, кодируем текст файла и добавляем в тело письма
        $body .= "Content-Type: application/octet-stream; name==?utf-8?B?".base64_encode($filename)."?=\n";
        $body .= "Content-Transfer-Encoding: base64\n";
        $body .= "Content-Disposition: attachment; filename==?utf-8?B?".base64_encode($filename)."?=\n\n";
        $body .= chunk_split(base64_encode($text))."\n";
        $body .= "--".$boundary."--\n";
        //Отправляем письма
        $mail = true;
        foreach ($arReceivers as $to) {
            $mail = mail($to, $subject, $body, $headers);
            if (!$mail) {
                $responseData->AddError('Возникла проблема с отправкой письма на адрес '.$to);
            }
        }

        if ($request && $mail) {
            $responseData->setSuccess();
            $responseData->AddMessage('Запрос успешно отправлен в банк', 'success');
            $connector->UpdateCertificateCreateDate();
        }
        ob_get_clean();

        return $responseData;
    }

    //TODO:
    private function DecryptPrivateCertificate()
    {
        $certsDir = realpath(dirname(__FILE__).'/ubrr.core/certs/');

        $privateCert = $certsDir.'/_new.key';
        $newPrivateCert = $certsDir.'/49909401_private.pem';
        $privatePass = '123456';
        $command = 'openssl rsa -in '.escapeshellarg($privateCert).' -outform PEM -out '.escapeshellarg(
                $newPrivateCert
            ).' -passin pass:'.escapeshellarg($privatePass);

        $result = false;
        $privateModulus = '';
        exec($command, $privateModulus, $result);
    }

    public function getRequestFilePath()
    {
        return Helpers::GetUbrrCoreDir().'/certs/request.csr';
    }

    public function getNewPrivateCertificateFilePath()
    {
        $privateKeyPath = Helpers::GetUbrrCoreDir().'/certs/_new.key';

        return $privateKeyPath;
    }

    public function getPrivateCertificateFilePath()
    {
        $validNames = $this->gatewayPanel->Certificates_DefaultPrivateFiles();

        return $validNames[0];
    }

    public function getPublicCertificateFilePath()
    {
        $validNames = $this->gatewayPanel->Certificates_DefaultPublicFiles();

        return $validNames[0];
    }

    public function getNewPublicCertificateFilePath()
    {
        $shopId = $this->gatewayPanel->getShopId();

        $certificatesDir = Helpers::GetUbrrCoreDir().'/certs/';

        $newPublicCertificateFilePath = $certificatesDir.'_new_'.$shopId.'.pem';

        return $newPublicCertificateFilePath;
    }

    public function CheckCertificatesModulus()
    {

        $this->gatewayPanel->Log('Проверка сертификатов:');

        $responseData = new DataContainer(true);

        $publicCertificate = $this->getNewPublicCertificateFilePath();
        $privateCertificate = $this->getNewPrivateCertificateFilePath();
        $privatePass = $this->gatewayPanel->GetCMSConnector()->GetNewPrivateCertificatePassword();

        $getPublicModulusCommand = 'openssl x509 -noout -modulus -in '.escapeshellarg($publicCertificate);
        $result = false;
        $publicModulus = '';
        exec($getPublicModulusCommand, $publicModulus, $result);
        $this->gatewayPanel->Log('Публичный:'."\t".$publicModulus[0]);

        if (!file_exists($privateCertificate)) {
            //TODO: Проверить лежат ли рабочие сертификаты на месте - тогда можно сообщить о том что сертификаты уже загружены
            $responseData->AddError('Файл приватного ключа '.$privateCertificate.' не найден!');
        } else {
            $getPrivateModulusCommand = 'openssl rsa -noout -modulus -in '.escapeshellarg(
                    $privateCertificate
                ).' -passin pass:'.escapeshellarg($privatePass);
            $result = false;
            $privateModulus = '';
            exec($getPrivateModulusCommand, $privateModulus, $result);
            $this->gatewayPanel->Log('Приватный:'."\t".$privateModulus[0]);

            if ($publicModulus != $privateModulus) {
                $responseData->AddError(
                    'Публичный сертификат не соответсвтует приватному. Использование не возможно. Проверьте правильный и публичный ключ Вы использовали.'
                );
            }
        }

        return $responseData;
    }

    public function TestAndReplaceWithNewCertificates()
    {
        $publicCertificate = $this->getNewPublicCertificateFilePath();
        $privateCertificate = $this->getNewPrivateCertificateFilePath();
        $privatePass = $this->gatewayPanel->GetCMSConnector()->GetNewPrivateCertificatePassword();

        $this->gatewayPanel->ServerConnectionConfig(true, $publicCertificate, $privateCertificate, $privatePass);

        $responseData = $this->CheckCertificatesModulus();

        if ($responseData->isSuccess()) {
            $responseData = $this->gatewayPanel->PrepareToPay(); //->GetJournal();

            if (!$responseData->isSuccess()) {
                $responseData->AddError(
                    'Не удалось исползовать сертификаты для взаимодействия с платежным шлюзом. Свяжитесь с банком или попробуйте перевыпустить сертификат.'
                );
            } else {
                //Заменяем старые сертификаты
                if (!rename($privateCertificate, $this->getPrivateCertificateFilePath())) {
                    $responseData->AddError('Не удалось скопировать новый приватный ключ');
                }

                if (!rename($publicCertificate, $this->getPublicCertificateFilePath())) {
                    $responseData->AddError('Не удалось скопировать новый публичный ключ');
                }

                //Удаляем файл запроса (если его не удалили руками)
                if (file_exists($this->getRequestFilePath())) {
                    unlink($this->getRequestFilePath());
                }

                //При копировании не было ошибок
                if ($responseData->isSuccess()) {
                    //Устанавливаем новый пароль приватного ключа
                    $this->gatewayPanel->GetCMSConnector()->SetPrivateCertificatePassword($privatePass);
                }
            }
        }

        return $responseData;
    }
}