<?php
include_once $_SERVER['DOCUMENT_ROOT'].'/wp-load.php';

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}
//include_once (  ABSPATH.'wp-content/plugins/woocommerce/includes/abstracts/abstract-wc-payment-gateway.php' );

/**
 * Платежный модуль УБРиР.
 *
 *
 * @class        WC_Gateway_UBRR
 * @extends        WC_Payment_Gateway
 * @version        1.0.0
 * @package        WooCommerce/Classes/Payment
 * @author        WooThemes
 */
class WC_Gateway_UBRR extends WC_Payment_Gateway
{

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->id = 'ubrr';
        $this->method_title = __('Платежный модуль УБРиР', 'woocommerce');
        $this->method_description = __('Оплата через УБРиР.', 'woocommerce');
        $this->has_fields = false;

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        $this->title = 'Оплата банковской картой';
        // Get settings
//        $this->title              = $this->get_option( 'Заголовок' );
//        $this->description        = $this->get_option( 'Описание' );
//        $this->instructions       = $this->get_option( 'Инструкции' );
        $this->enable_for_methods = $this->get_option('enable_for_methods', array());
        $this->enable_for_virtual = $this->get_option('enable_for_virtual', 'yes') === 'yes' ? true : false;

        add_action('woocommerce_update_options_payment_gateways_'.$this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_cod', array($this, 'thankyou_page'));

        // Customer Emails
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
    }

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $shipping_methods = array();

        if (is_admin()) {
            foreach (WC()->shipping()->load_shipping_methods() as $method) {
                $shipping_methods[$method->id] = $method->get_method_title();
            }
        }

        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Включить платежную систему', 'woocommerce'),
                'label' => __('Включить платежную систему', 'woocommerce'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
//            'title' => array(
//                'title'       => __( 'Title', 'woocommerce' ),
//                'type'        => 'text',
//                'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce' ),
//                'default'     => __( 'Cash on Delivery', 'woocommerce' ),
//                'desc_tip'    => true,
//            ),
//            'description' => array(
//                'title'       => __( 'Description', 'woocommerce' ),
//                'type'        => 'textarea',
//                'description' => __( 'Payment method description that the customer will see on your website.', 'woocommerce' ),
//                'default'     => __( 'Pay with cash upon delivery.', 'woocommerce' ),
//                'desc_tip'    => true,
//            ),
//            'instructions' => array(
//                'title'       => __( 'Instructions', 'woocommerce' ),
//                'type'        => 'textarea',
//                'description' => __( 'Instructions that will be added to the thank you page.', 'woocommerce' ),
//                'default'     => __( 'Pay with cash upon delivery.', 'woocommerce' ),
//                'desc_tip'    => true,
//            ),
            'enable_for_methods' => array(
                'title' => __('Enable for shipping methods', 'woocommerce'),
                'type' => 'multiselect',
                'class' => 'wc-enhanced-select',
                'css' => 'width: 450px;',
                'default' => '',
                'description' => __(
                    'If COD is only available for certain methods, set it up here. Leave blank to enable for all methods.',
                    'woocommerce'
                ),
                'options' => $shipping_methods,
                'desc_tip' => true,
                'custom_attributes' => array(
                    'data-placeholder' => __('Select shipping methods', 'woocommerce'),
                ),
            ),
            'enable_for_virtual' => array(
                'title' => __('Accept for virtual orders', 'woocommerce'),
                'label' => __('Accept COD if the order is virtual', 'woocommerce'),
                'type' => 'checkbox',
                'default' => 'yes',
            ),
        );
    }

    /**
     * Check If The Gateway Is Available For Use.
     *
     * @return bool
     */
    public function is_available()
    {
        $order = null;
        $needs_shipping = false;

        // Test if shipping is needed first
        if (WC()->cart && WC()->cart->needs_shipping()) {
            $needs_shipping = true;
        } elseif (is_page(wc_get_page_id('checkout')) && 0 < get_query_var('order-pay')) {
            $order_id = absint(get_query_var('order-pay'));
            $order = wc_get_order($order_id);

            // Test if order needs shipping.
            if (0 < sizeof($order->get_items())) {
                foreach ($order->get_items() as $item) {
                    $_product = $order->get_product_from_item($item);
                    if ($_product && $_product->needs_shipping()) {
                        $needs_shipping = true;
                        break;
                    }
                }
            }
        }

        $needs_shipping = apply_filters('woocommerce_cart_needs_shipping', $needs_shipping);

        // Virtual order, with virtual disabled
        if (!$this->enable_for_virtual && !$needs_shipping) {
            return false;
        }

        // Check methods
        if (!empty($this->enable_for_methods) && $needs_shipping) {

            // Only apply if all packages are being shipped via chosen methods, or order is virtual
            $chosen_shipping_methods_session = WC()->session->get('chosen_shipping_methods');

            if (isset($chosen_shipping_methods_session)) {
                $chosen_shipping_methods = array_unique($chosen_shipping_methods_session);
            } else {
                $chosen_shipping_methods = array();
            }

            $check_method = false;

            if (is_object($order)) {
                if ($order->shipping_method) {
                    $check_method = $order->shipping_method;
                }

            } elseif (empty($chosen_shipping_methods) || sizeof($chosen_shipping_methods) > 1) {
                $check_method = false;
            } elseif (sizeof($chosen_shipping_methods) == 1) {
                $check_method = $chosen_shipping_methods[0];
            }

            if (!$check_method) {
                return false;
            }

            $found = false;

            foreach ($this->enable_for_methods as $method_id) {
                if (strpos($check_method, $method_id) === 0) {
                    $found = true;
                    break;
                }
            }

            if (!$found) {
                return false;
            }
        }

        return parent::is_available();
    }

    private static function woo_commerce_version3x_check($version = '3.0')
    {
        if (class_exists('WooCommerce')) {
            global $woocommerce;
            if (version_compare($woocommerce->version, $version, ">=")) {
                return true;
            }
        }

        return false;
    }

    /**
     * Process the payment and return the result.
     *
     * @param int $orderId
     * @return array
     */
    public function process_payment($orderId)
    {
        include_once $_SERVER['DOCUMENT_ROOT'].'/wp-content/plugins/ubrr/Wordpress.php';

        global $woocommerce;
        $order = wc_get_order($orderId);

        // Mark as processing or on-hold (payment won't be taken until delivery)
        $order->update_status(get_option('ubrr_payment_setting_init_woo'));

        if ($this->woo_commerce_version3x_check()) {
            wc_reduce_stock_levels($orderId);
        } else {
            $order->reduce_order_stock();
        }

        $wooSessionID = $woocommerce->session->get_customer_id();
        $amount = $woocommerce->cart->total;
        //Проверяем включен ли отложенный платеж
        if (get_option('post_payment')) {
            $redirect_url = get_site_url(
                ).'/wp-content/plugins/ubrr/wp_shop_plugins/woocommerce/woo-choose.php?id='.$wooSessionID.'&amount='.$amount.'&orderid='.$orderId;

            return array(
                'result' => 'success',
                'redirect' => $redirect_url,
            );
        } else {

            $site_url = get_site_url();
            $callbackURL = $site_url.'/wp-content/plugins/ubrr/callback/woo_callback.php';

            $cmsModule = new UBRIR\Wordpress();
            //Заполняем описание в TWPG
            $description = 'Плательщик: '.$order->get_billing_first_name().' '.$order->get_billing_last_name().
                ' email: '.$order->get_billing_email().
                ' Телефон: '.$order->get_billing_phone();
            if ($order->get_customer_note()) {
                $description .= ' Дополнительно: '.$order->get_customer_note();
            }
            $cmsModule->setOrderDescription($description);

            //Подготавливаем почву для оплаты
            $response = $cmsModule->SetupAndPrepareToPay($orderId, $amount, $callbackURL);
            // Remove cart
            WC()->cart->empty_cart();

            if ($response->isSuccess()) {
                $TWPG_OrderId = $response->GetTWPG_OrderId();
                $SessionId = $response->GetSessionId();

                $cmsModule->SaveTWPGOrder($orderId, $TWPG_OrderId, $SessionId); //, $ecomSessionID

                $redirect_url = $response->GetPaymentUrl($TWPG_OrderId, $SessionId);

                return array(
                    'result' => 'success',
                    'redirect' => $redirect_url,
                );

            } else {
                //Не удалось подготовиться как следует
                echo 'Ошибка связи с сервером оплаты, обратитесь к администратору. <br><a href="/">Вернуться на сайт</a>';
            }
        }
    }

//    /**
//     * Output for the order received page.
//     */
//    public function thankyou_page() {
//        if ( $this->instructions ) {
//            echo wpautop( wptexturize( $this->instructions ) );
//        }
//    }

//    /**
//     * Add content to the WC emails.
//     *
//     * @access public
//     * @param WC_Order $order
//     * @param bool $sent_to_admin
//     * @param bool $plain_text
//     */
//    public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
//        if ( $this->instructions && ! $sent_to_admin && 'cod' === $order->payment_method ) {
//            echo wpautop( wptexturize( $this->instructions ) ) . PHP_EOL;
//        }
//    }
}
